/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2013  Ciaran Anscomb
 *
 *  See COPYING.GPL for redistribution conditions. */

#ifndef XROAR_VDG_H_
#define XROAR_VDG_H_

#include <stdint.h>

// Horizontal timing, all measured in half-VDG-clocks (i.e., pixels)

#define VDG_tFP   (17)  // 14
#define VDG_tWHS  (32)  // 35
#define VDG_tBP   (35)
#define VDG_tHBNK (VDG_tFP + VDG_tWHS + VDG_tBP)
#define VDG_tLB   (60)  // 59
#define VDG_tAV   (256)
#define VDG_tRB   (56)
#define VDG_tAVB  (VDG_tLB + VDG_tAV + VDG_tRB)
#define VDG_tHST  (VDG_tHBNK + VDG_tAVB)
// tHCD = time from start of back porch to beginning of colour burst
#define VDG_tHCD  (7)
// tCB = duration of colour burst
#define VDG_tCB   (21)

/* All horizontal timings shall remain relative to the HS pulse falling edge */
#define VDG_HS_FALLING_EDGE    (0)
#define VDG_HS_RISING_EDGE     (VDG_HS_FALLING_EDGE + VDG_tWHS)
#define VDG_LEFT_BORDER_START  (VDG_HS_FALLING_EDGE + VDG_tWHS + VDG_tBP)
#define VDG_ACTIVE_LINE_START  (VDG_LEFT_BORDER_START + VDG_tLB)
#define VDG_RIGHT_BORDER_START (VDG_ACTIVE_LINE_START + VDG_tAV)
#define VDG_RIGHT_BORDER_END   (VDG_RIGHT_BORDER_START + VDG_tRB)
#define VDG_LINE_DURATION      (VDG_tHBNK + VDG_tAVB)
#define VDG_PAL_PADDING_LINE   VDG_LINE_DURATION

#define VDG_VBLANK_START       (0)
#define VDG_TOP_BORDER_START   (VDG_VBLANK_START + 13)
#define VDG_ACTIVE_AREA_START  (VDG_TOP_BORDER_START + 25)
#define VDG_ACTIVE_AREA_END    (VDG_ACTIVE_AREA_START + 192)
#define VDG_BOTTOM_BORDER_END  (VDG_ACTIVE_AREA_END + 26)
#define VDG_VRETRACE_END       (VDG_BOTTOM_BORDER_END + 6)
#define VDG_FRAME_DURATION     (262)

/* Basic colours the VDG can generate */

enum vdg_colour {
	VDG_GREEN, VDG_YELLOW, VDG_BLUE, VDG_RED,
	VDG_WHITE, VDG_CYAN, VDG_MAGENTA, VDG_ORANGE,
	VDG_BLACK, VDG_DARK_GREEN, VDG_DARK_ORANGE, VDG_BRIGHT_ORANGE
};

/* External handler to fetch data for display.  First arg is number of bytes,
 * second a pointer to a buffer to receive them. */
extern void (*vdg_fetch_bytes)(int, uint8_t *);

/* Delegates for signalling HS/FS fall/rise */
typedef struct {
	void (*delegate)(void *, _Bool level);
	void *dptr;
} vdg_edge_delegate;

extern vdg_edge_delegate vdg_signal_hs;
extern vdg_edge_delegate vdg_signal_fs;

// Video module configures these window coordinates.  x1 = pixels from left
// border, y1 = scanlines from top border.  x2 must be within the right border,
// y2 within the bottom border.

extern unsigned vdg_window_x1, vdg_window_y1;
extern unsigned vdg_window_x2, vdg_window_y2;

// Set to emulate 6847T1.
extern _Bool vdg_t1;

void vdg_init(void);
void vdg_reset(void);

/* Mode bits:
 * 7..5  GM2..GM0
 * 4    ¬INT/EXT
 * 3    CSS */
void vdg_set_mode(unsigned mode);

#endif  /* XROAR_VDG_H_ */
